==> Rbeast_0.9.1.tar.gz: Rbeast/src/beastv2_func.h <==
#pragma once
#include <inttypes.h>  
#include "abc_common.h"
#include "abc_ide_util.h"
#include "abc_mem.h"
#include "abc_blas_lapack_lib.h"
#include "beastv2_header.h"
extern void SetupPointersForCoreResults(CORESULT* coreResults,BEAST2_BASIS_PTR b,I32 NumBasis,BEAST2_RESULT* resultChain);
extern void CvtKnotsToBinVec(BEAST2_BASIS_PTR b,I32 NUMBASIS,I32 N,BEAST2_YINFO_PTR yInfo);
extern void GenarateRandomBasis(BEAST2_BASIS_PTR basis,I32 NUMBASIS,I32 N,BEAST2_RNDSTREAM* PRAND);
void BEAST2_EvaluateModel(
    BEAST2_MODELDATA* curmodel,BEAST2_BASIS_PTR b,F32PTR Xt_mars,I32 N,I32 NUMBASIS,
    BEAST2_YINFO_PTR  pyInfo,BEAST2_HyperPar*hyperPar,F32 precVal,VOID_PTR stream);
void MatxVec(BEAST2_BASESEG* SEG,I32 numSeg,F32PTR X,F32PTR Y,F32PTR XtY,I32 N);
I32  GetInfoBandList(BEAST2_BASESEG* info,BEAST2_MODEL_PTR model,I32 Klastcol);
I32  GetInfoBandList_post(BEAST2_BASESEG* info,BEAST2_MODEL_PTR model,I32 Kstartcol);
void MatxMat(BEAST2_BASESEG* infoX,I32 numBandsX,F32PTR X,BEAST2_BASESEG* infoY,
             I32 numBandsY,F32PTR Y,F32PTR XtY,I32 N,I32 Knew);
void XtX_ByGroup(BEAST2_BASESEG* SEG,I32 numSeg,F32PTR X,F32PTR XtX,I32 N,I32 Knew);
static INLINE int GetTotalColNumOfXmars(BEAST2_BASIS_PTR b,I32 NUM_BASIS) {
    I32 K;
    if      (NUM_BASIS==1)     K=b[0].K;
    else if (NUM_BASIS==2)     K=b[0].K+b[1].K;
    else if (NUM_BASIS==3)     K=b[0].K+b[1].K+b[2].K;
    else   {
        K=0;
        for (int i=0; i < NUM_BASIS; i++)    K+=b[i].K;
    }
    return K;
}
static INLINE void UpdateBasisKbase(BEAST2_BASIS_PTR b,I32 NUMBASIS,int startBasisIdx) {
    if (NUMBASIS==1) 
        b[0].Kbase=0;    
    else if (NUMBASIS==2) {
        if (startBasisIdx==0) {
              b[1].Kbase=b[0].K;
        } 
        else  {            
        }
    } else if (NUMBASIS==3) {
        if (startBasisIdx==0) {
            b[1].Kbase=b[0].K;
            b[2].Kbase=b[1].Kbase+b[1].K;
        } else if (startBasisIdx==1) {
            b[2].Kbase=b[1].Kbase+b[1].K;
        } else if (startBasisIdx==2) {
        }          
    }  else if (NUMBASIS > 3)  {           
        for (rI32 i=startBasisIdx+1; i < MAX_NUM_BASIS; i++) {
            b[i].Kbase=b[i-1].Kbase+b[i-1].K;
        }   
    }
}
 void MoveCOLsWithinMatrix(F32PTR X,I32 N,I32 Kstart,I32 Kend,I32 Knewstart);
 static void MatxVec_FULL(BEAST2_BASESEG* SEG,I32 numSeg,F32PTR X,F32PTR Y,F32PTR XtY,I32 N)
{
	 I32 Kterms=0;
	 for (int i=0; i < numSeg; i++) {			 Kterms+=SEG[i].K; 	 }
	 I32 Npad=(N+7)/8 * 8;
	r_cblas_sgemv(CblasColMajor,CblasTrans,
		N,Kterms,1.f,
		X,Npad,
		Y,1L,0.f,
		XtY,1L);	
}
 static  void MatxMat_FULL(BEAST2_BASESEG*infoX,I32 numBandsX,F32PTR X,BEAST2_BASESEG*infoY,I32 numBandsY,F32PTR Y,F32PTR XtY,I32 N,I32 Knew)
{
	I32 Kterms1=0;
	I32 Kterms2=0;
	for (int i=0; i < numBandsX; i++) { Kterms1+=infoX[i].K; }
	for (int i=0; i < numBandsY; i++) { Kterms2+=infoY[i].K; }
	I32 Npad=(N+7)/8 * 8;
	r_cblas_sgemm(CblasColMajor,CblasTrans,CblasNoTrans,\
		Kterms1,Kterms2,N,1.0f,\
		X,Npad,
		Y,Npad,0.f,\
		XtY,Knew); 
}
static void XtX_ByGroup_FULL(BEAST2_BASESEG* SEG,I32 numSeg,F32PTR X,F32PTR XtX,I32 N,I32 Knew) {
	I32 Kterms=0;
	for (int i=0; i < numSeg; i++) { Kterms+=SEG[i].K; }
	I32 Npad=(N+7)/8 * 8;
	r_cblas_sgemm(CblasColMajor,CblasTrans,CblasNoTrans,
					Kterms,Kterms,N,1.0,
					X,Npad,
					X,Npad,0.,
					XtX,Knew);
}
void MR_EvaluateModel(
	BEAST2_MODELDATA* curmodel,BEAST2_BASIS_PTR b,F32PTR Xt_mars,I32 N,I32 NUMBASIS,
	BEAST2_YINFO_PTR pyInfo,BEAST2_HyperPar*hyperPar,F32 precVal,VOID_PTR stream);

==> Rbeast_0.9.2.tar.gz: Rbeast/src/beastv2_func.h <==
#pragma once
#include <inttypes.h>  
#include "abc_common.h"
#include "abc_ide_util.h"
#include "abc_mem.h"
#include "abc_blas_lapack_lib.h"
#include "beastv2_header.h"
extern int  GetMedianNcp(F32PTR prob,I32 N);
extern void SetupPointersForCoreResults(CORESULT* coreResults,BEAST2_BASIS_PTR b,I32 NumBasis,BEAST2_RESULT* resultChain);
extern void CvtKnotsToBinVec(BEAST2_BASIS_PTR b,I32 NUMBASIS,I32 N,BEAST2_YINFO_PTR yInfo);
extern void GenarateRandomBasis(BEAST2_BASIS_PTR basis,I32 NUMBASIS,I32 N,BEAST2_RNDSTREAM* PRAND);
void BEAST2_EvaluateModel(
    BEAST2_MODELDATA* curmodel,BEAST2_BASIS_PTR b,F32PTR Xt_mars,I32 N,I32 NUMBASIS,
    BEAST2_YINFO_PTR  pyInfo,BEAST2_HyperPar*hyperPar,F32 precVal,VOID_PTR stream);
void MatxVec(BEAST2_BASESEG* SEG,I32 numSeg,F32PTR X,F32PTR Y,F32PTR XtY,I32 N);
I32  GetInfoBandList(BEAST2_BASESEG* info,BEAST2_MODEL_PTR model,I32 Klastcol);
I32  GetInfoBandList_post(BEAST2_BASESEG* info,BEAST2_MODEL_PTR model,I32 Kstartcol);
void MatxMat(BEAST2_BASESEG* infoX,I32 numBandsX,F32PTR X,BEAST2_BASESEG* infoY,
             I32 numBandsY,F32PTR Y,F32PTR XtY,I32 N,I32 Knew);
void XtX_ByGroup(BEAST2_BASESEG* SEG,I32 numSeg,F32PTR X,F32PTR XtX,I32 N,I32 Knew);
static INLINE int GetTotalColNumOfXmars(BEAST2_BASIS_PTR b,I32 NUM_BASIS) {
    I32 K;
    if      (NUM_BASIS==1)     K=b[0].K;
    else if (NUM_BASIS==2)     K=b[0].K+b[1].K;
    else if (NUM_BASIS==3)     K=b[0].K+b[1].K+b[2].K;
    else   {
        K=0;
        for (int i=0; i < NUM_BASIS; i++)    K+=b[i].K;
    }
    return K;
}
static INLINE void UpdateBasisKbase(BEAST2_BASIS_PTR b,I32 NUMBASIS,int startBasisIdx) {
    if (NUMBASIS==1) 
        b[0].Kbase=0;    
    else if (NUMBASIS==2) {
        if (startBasisIdx==0) {
              b[1].Kbase=b[0].K;
        } 
        else  {            
        }
    } else if (NUMBASIS==3) {
        if (startBasisIdx==0) {
            b[1].Kbase=b[0].K;
            b[2].Kbase=b[1].Kbase+b[1].K;
        } else if (startBasisIdx==1) {
            b[2].Kbase=b[1].Kbase+b[1].K;
        } else if (startBasisIdx==2) {
        }          
    }  else if (NUMBASIS > 3)  {           
        for (rI32 i=startBasisIdx+1; i < MAX_NUM_BASIS; i++) {
            b[i].Kbase=b[i-1].Kbase+b[i-1].K;
        }   
    }
}
 void MoveCOLsWithinMatrix(F32PTR X,I32 N,I32 Kstart,I32 Kend,I32 Knewstart);
 static void MatxVec_FULL(BEAST2_BASESEG* SEG,I32 numSeg,F32PTR X,F32PTR Y,F32PTR XtY,I32 N)
{
	 I32 Kterms=0;
	 for (int i=0; i < numSeg; i++) {			 Kterms+=SEG[i].K; 	 }
	 I32 Npad=(N+7)/8 * 8; Npad=N;
	r_cblas_sgemv(CblasColMajor,CblasTrans,
		N,Kterms,1.f,
		X,Npad,
		Y,1L,0.f,
		XtY,1L);	
}
 static  void MatxMat_FULL(BEAST2_BASESEG*infoX,I32 numBandsX,F32PTR X,BEAST2_BASESEG*infoY,I32 numBandsY,F32PTR Y,F32PTR XtY,I32 N,I32 Knew)
{
	I32 Kterms1=0;
	I32 Kterms2=0;
	for (int i=0; i < numBandsX; i++) { Kterms1+=infoX[i].K; }
	for (int i=0; i < numBandsY; i++) { Kterms2+=infoY[i].K; }
	I32 Npad=(N+7)/8 * 8; Npad=N;
	r_cblas_sgemm(CblasColMajor,CblasTrans,CblasNoTrans,\
		Kterms1,Kterms2,N,1.0f,\
		X,Npad,
		Y,Npad,0.f,\
		XtY,Knew); 
}
static void XtX_ByGroup_FULL(BEAST2_BASESEG* SEG,I32 numSeg,F32PTR X,F32PTR XtX,I32 N,I32 Knew) {
	I32 Kterms=0;
	for (int i=0; i < numSeg; i++) { Kterms+=SEG[i].K; }
	I32 Npad=(N+7)/8 * 8; Npad=N;
	r_cblas_sgemm(CblasColMajor,CblasTrans,CblasNoTrans,
					Kterms,Kterms,N,1.0,
					X,Npad,
					X,Npad,0.,
					XtX,Knew);
}
void MR_EvaluateModel(
	BEAST2_MODELDATA* curmodel,BEAST2_BASIS_PTR b,F32PTR Xt_mars,I32 N,I32 NUMBASIS,
	BEAST2_YINFO_PTR pyInfo,BEAST2_HyperPar*hyperPar,F32 precVal,VOID_PTR stream);
